//toimivat 3.1-3.12


const http = require('http')
const express = require('express')
const app = express()

const Contact = require('./models/contact')

const bodyParser = require('body-parser')
app.use(bodyParser.json())

const cors = require('cors')
app.use(cors())

const logger = (request, response, next) => {
  console.log('Method:', request.method)
  console.log('Path:  ', request.path)
  console.log('Body:  ', request.body)
  console.log('---')
  next()
}
app.use(express.static('build'))

app.use(logger)

const formatContact = (contact) => {
  return {
    name: contact.name,
    number: contact.number,
    id: contact._id
  }
}

let persons = [
  {
    name: "Murto Karhunen",
    number: "040-123456",
    id: 1
  },
  {
    name: "Lotto Massinen",
    number: "040-123456",
    id: 2
  },
  {
    name: "Valo Pää",
    number: "040-123456",
    id: 3
  },
  {
    name: "Erkka Markkanen",
    number: "040-123456",
    id: 4
  }
]
app.get('/', (req, res) => {
  res.send('<h1>please move to /api/persons</h1>')
})

app.get('/api/persons', (request, response) => {
  Contact
    .find({})
    .then(persons => {
      response.json(persons.map(formatContact))
    })
})

app.get('/api/persons/:id', (request, response) => {
  Contact
  .findById(request.params.id)
  .then(contact => {
    if (contact){
    response.json(formatContact(contact))
  } else{
    response.status(404).end()
  }
  })
  .catch(error => {
      console.log(error)
      response.status(404).send({error: 'Malformatted id'})
    })
})

app.post('/api/persons', (request, response) => {
  const body = request.body
  const person_list = () => persons.map(part => part.name)

  if ((body.name === undefined) || (body.number === undefined)) {
    return response.status(400).json({error: 'Please check the input, there is some content missing.'})
  }

  if ((person_list().includes(body.name))){
    return response.status(400).json({error: 'Sorry, but the name must be unique.'})
    console.log("name must be unique!");
  }

  const contact = new Contact ({
    name: body.name,
    number: body.number
  })

  contact
    .save()
    .then(formatContact)
    .then(savedAndFormattedContact => {
      response.json(savedAndFormattedContact)
    })
})

app.delete('/api/persons/:id' , (request, response) =>{
  Contact
      .findByIdAndRemove(request.params.id)
      .then(result => {
        response.status(204).end()
      })
      .catch(error => {
        response.status(400).send({ error: 'malformatted id' })
      })
})

const generateId = () => {
  const minId = Math.ceil(persons.length > 0 ? persons.map(n => n.id).sort((a,b) => a - b).reverse()[0] : 1)
  const maxId = Math.floor(1000)
  return Math.floor(Math.random() * (maxId - minId));
}

const error = (request, response) => {
  response.status(404).send({error: 'unknown endpoint'})
}

app.use(error)

const PORT = process.env.PORT || 3001
app.listen(PORT, () => {
  console.log(`Server running on port ${PORT}`)
})
